import Methods
import matplotlib.pyplot as plt
import numpy as np
from mpl_toolkits.mplot3d import Axes3D


def material_thickness_example(initial_point=10):
    """
    Function for demonstrating the use of different functions in
    Methods.
    :param initial_point: The initial point, default is 10.
    """
    lower_bound = Methods.generate_points_disk(20, 1, 0)
    upper_bound = Methods.generate_points_disk(20, 1, 2)
    all_points = np.concatenate((lower_bound, upper_bound), axis=0)
    general_kd_tree = Methods.kd_tree(all_points)

    number_of_nearest_neighbours = int(Methods.neighborhood_selection(
        general_kd_tree, all_points, initial_point))
    distances, nearest_neighbours_index = general_kd_tree.query(
        [all_points[initial_point]], k=number_of_nearest_neighbours)
    nearest_neighbours_points = Methods.flatten_2d(
        all_points[nearest_neighbours_index])
    # *least_squares_plane, least_squares_normal =
    # Methods.generate_plane_lstsq(nearest_neighbours_points)
    # *least_squares_plane, least_squares_normal =
    # Methods.generate_plane_pca(nearest_neighbours_points)
    *least_squares_plane, least_squares_normal = \
        Methods.generate_plane_svd(nearest_neighbours_points)
    reduced_points = np.delete(all_points, [nearest_neighbours_index], 0)
    smallest_angle_point = Methods.brute_optimal_angle(
        reduced_points, all_points[initial_point], least_squares_normal)
    euclidean_distance = Methods.euclidean_distance(
        all_points[initial_point], smallest_angle_point)

    plt3d = plt.figure(figsize=(8, 8)).gca(projection='3d')
    plt3d.set_xlim3d(-1.5, 1.5)
    plt3d.set_ylim3d(-1.5, 1.5)
    plt3d.set_zlim3d(-0.5, 2.5)
    plt3d.set_xlabel('$X$', fontsize=20)
    plt3d.set_ylabel('$Y$', fontsize=20)
    plt3d.set_zlabel('$Z$', fontsize=20)
    plt3d.set_xticks([])
    plt3d.set_yticks([])
    plt3d.set_zticks([])
    plt3d.view_init(elev=20, azim=80)
    plt3d.scatter(*nearest_neighbours_points.T, c='b', marker='o',
    alpha=.5, label='{} Evaluated points from the initial point: '
                    '{}'.format(number_of_nearest_neighbours,
                                    initial_point))
    plt3d.plot_surface(*least_squares_plane, color='g', alpha=.2,
                       linewidth=0, zorder=1)
    plt3d.scatter(*smallest_angle_point.T, color='b')
    plt3d.scatter(*smallest_angle_point.T, color='y', label=
    'Closest angle intersection point: {}'.format(
        np.where(all_points == smallest_angle_point)[0][0]))
    matrix = np.asarray(list(zip(
        all_points[initial_point],  smallest_angle_point)))
    plt3d.plot3D(*matrix, color='b', alpha=.2, label=
    'Material thickness: {0:.4f}'.format(euclidean_distance))
    dataset_removed = np.delete(reduced_points,
    [np.where(reduced_points == smallest_angle_point)[0][0],
     np.where(reduced_points == smallest_angle_point)[0][0]], 0)
    plt3d.scatter(*dataset_removed.T, color='r', alpha=.5, label=
    'All other points')
    plt.legend(prop={"size": 16})
    plt.show()


if __name__ == '__main__':
    material_thickness_example()
